#include <iostream>
#include <cstring>
#include <bitset>
#include <unistd.h>
#include <time.h>
#include <csignal>
#include <cstdlib>
#include "../include/HCNetSDK.h"
#include "../include/HkStreamer.h"
#include "../include/convert.h"

using namespace std;
long playHandle = -1;
bool is_playback = false;
long lUserID;
const int outcache_size=10240;

void signalexit(int signal){
    if(playHandle!= -1){
        if(is_playback){
            NET_DVR_StopGetFile(playHandle);
        } else {
            NET_DVR_StopRealPlay(playHandle);
        }
        NET_DVR_Logout(lUserID);
        NET_DVR_Cleanup();
    }
    exit(signal);
}

void exitHandler(){
    signalexit(0);
}


template<typename T>
void CALLBACK fPlayDataCallBack(LONG lPlayHandle, DWORD dwDataType, BYTE *pBuffer, DWORD dwBufSize, T dwUser){
    if(dwDataType == NET_DVR_SYSHEAD){
        return;
    }
    char* p= (char*)pBuffer;
    cout.write(p, dwBufSize);
    cerr<<"recv:"<<dwBufSize<<endl;
    cout.flush();

}

bool checkResult(bool result, string msg){
  if(result){
    cerr<<msg<<":"<<NET_DVR_GetLastError()<<endl;
    return true;
  }
  return false;
}

int login(LPLoginInfo loginInfo)
{
  NET_DVR_Init();
  long lUserID;
  // login
  NET_DVR_USER_LOGIN_INFO struLoginInfo = {0};
  NET_DVR_DEVICEINFO_V40 struDeviceInfoV40 = {0};
  struLoginInfo.bUseAsynLogin = false;

  struLoginInfo.wPort = 8000;
  memcpy(struLoginInfo.sDeviceAddress, loginInfo->sDeviceAddress.c_str(), NET_DVR_DEV_ADDRESS_MAX_LEN);
  memcpy(struLoginInfo.sUserName, loginInfo->sUserName.c_str(), NAME_LEN);
  memcpy(struLoginInfo.sPassword, loginInfo->sPassword.c_str(), NAME_LEN);
  std::cerr << struLoginInfo.sDeviceAddress << " " << struLoginInfo.sUserName << " " << struLoginInfo.sPassword << std::endl;
  lUserID = NET_DVR_Login_V40(&struLoginInfo, &struDeviceInfoV40);
  cerr<<"UserID:"<<lUserID<<endl;
  return lUserID;
}



int getCfg(LPLoginInfo loginInfo)
{
  lUserID = login(loginInfo);
  if (lUserID < 0)
  {
    std::cerr << "pyd1---Login error" << NET_DVR_GetLastError() << std::endl;
    return HPR_ERROR;
  }
  NET_DVR_IPPARACFG_V40 ipcfg;
  DWORD bytesReturned = 0;
  ipcfg.dwSize = sizeof(NET_DVR_IPPARACFG_V40);
  int iGroupNO = 0;
  bool resCode = NET_DVR_GetDVRConfig(lUserID, NET_DVR_GET_IPPARACFG_V40, iGroupNO, &ipcfg, sizeof(NET_DVR_IPPARACFG_V40), &bytesReturned);
  if (!resCode)
  {
    DWORD code = NET_DVR_GetLastError();
    std::cout << "NET_DVR_GetDVRConfig failed " << NET_DVR_GetErrorMsg((LONG *)(&code)) << std::endl;
    NET_DVR_Logout(lUserID);
    NET_DVR_Cleanup();
    return -1;
  }
  std::cout << "设备组 " << ipcfg.dwGroupNum << " 数字通道个数 " << ipcfg.dwDChanNum << " 起始通道 " << ipcfg.dwStartDChan << std::endl
            << std::endl;
  for (int i = 0; i < ipcfg.dwDChanNum; i++)
  {
    NET_DVR_PICCFG_V30 channelInfo;
    bytesReturned = 0;
    channelInfo.dwSize = sizeof(NET_DVR_PICCFG_V30);
    int channelNum = i + ipcfg.dwStartDChan;
    NET_DVR_GetDVRConfig(lUserID, NET_DVR_GET_PICCFG_V30, channelNum, &channelInfo, sizeof(NET_DVR_PICCFG_V30), &bytesReturned);
    std::cout << "通道号 " << channelNum << "\t通道名称 " << channelInfo.sChanName;
    std::cout << "\t用户名 " << ipcfg.struIPDevInfo[i].sUserName << "\t密码 " << ipcfg.struIPDevInfo[i].sPassword;
    std::cout << "\t设备ID " << (long)ipcfg.struIPDevInfo[i].szDeviceID;
    std::cout << "\tip地址 " << ipcfg.struIPDevInfo[i].struIP.sIpV4 << "\t端口 " << ipcfg.struIPDevInfo[i].wDVRPort << std::endl;
  }
  NET_DVR_Logout(lUserID);
  NET_DVR_Cleanup();
  return 0;
}

int _playback(long lUserID, int timezone,LPStream stream) {
    NET_DVR_VOD_PARA struVodPara = {0};
  convert_isotimestr_time(stream->start,timezone, &(struVodPara.struBeginTime));
  convert_isotimestr_time(stream->end,timezone, &(struVodPara.struEndTime));

  struVodPara.struIDInfo.dwChannel = stream->lChannel;
   playHandle = NET_DVR_PlayBackByTime_V40(lUserID, &struVodPara);
  if(checkResult(playHandle<0,"play back error")){
    return HPR_ERROR;
  }
  is_playback = true;
  char usrInfo[200];
  if(checkResult(!NET_DVR_SetPlayDataCallBack_V40(playHandle, fPlayDataCallBack<void *>, usrInfo), "setup callback error")){
    return HPR_ERROR;
  }

  if(checkResult(!NET_DVR_PlayBackControl_V40(playHandle, NET_DVR_PLAYSTART,NULL, 0, NULL, NULL),"start playback error")){
    return HPR_ERROR;
  }
  if(checkResult(!NET_DVR_PlayBackControl_V40(playHandle, NET_DVR_SETSPEED,&(stream->bitrate), 0, NULL, NULL),"set plackback speed error")){
    return HPR_ERROR;
  }
  int pos = 0;
  for (pos = 0; pos < 100 && pos >= 0; pos = NET_DVR_GetDownloadPos(playHandle))
  {
    // output progress in cerr
    std::cerr<<"Pos:"<<pos<<std::endl;
    sleep(10);
  }
  if(checkResult(!NET_DVR_StopGetFile(playHandle),"failed to stop playback")){
    return HPR_ERROR;
  }
  playHandle = -1;
  return HPR_OK;
}

int _readChannelConfig(long lUserID,LPStream stream){
  NET_DVR_IPPARACFG_V40 ipcfg;
  DWORD bytesReturned = 0;
  ipcfg.dwSize = sizeof(NET_DVR_IPPARACFG_V40);
  int iGroupNO = 0;
  bool resCode = NET_DVR_GetDVRConfig(lUserID, NET_DVR_GET_IPPARACFG_V40, iGroupNO, &ipcfg, sizeof(NET_DVR_IPPARACFG_V40), &bytesReturned);
  if(checkResult(!resCode, "read channel config fail")){
    return HPR_ERROR;
  }

  if(checkResult(stream->lChannel > ipcfg.dwDChanNum, "channel not correct")){
    return HPR_ERROR;
  }

  stream->lChannel = ipcfg.dwStartDChan + stream->lChannel -1;
  stream->bitrate = 2048; // controll speed for download, TODO: get bitrate from device;
  return 0;
}

void _installExitHandler(){
  atexit(exitHandler);
  signal(SIGINT,signalexit);
}


int playback(LPLoginInfo loginInfo, LPStream stream)
{
  lUserID = login(loginInfo);
  if(checkResult(lUserID<0, "device login error")){
    return HPR_ERROR;
  }
  // TODO: find timezone cfg
  int timezone = 8;
  int fChannelInfo = _readChannelConfig(lUserID, stream);
  if(fChannelInfo == HPR_ERROR){
    return fChannelInfo;
  }
//  stream->lChannel = stream->lChannel + 33/*channel start*/ - 1;
  int playbackResult = _playback(lUserID, timezone, stream);
  NET_DVR_Logout(lUserID);
  NET_DVR_Cleanup();
  return playbackResult;
}

int _getStream(long lUserID,LPStream stream){
  long lRealPlayHandle;
  NET_DVR_PREVIEWINFO struPlayInfo = {0};
   struPlayInfo.hPlayWnd     = 0;
  struPlayInfo.lChannel     = stream->lChannel;  //channel NO
   struPlayInfo.dwStreamType = 1;// 采用子码流
  struPlayInfo.dwLinkMode   = 0;
  struPlayInfo.bBlocked = 1;
  struPlayInfo.dwDisplayBufNum = 1;
  playHandle = NET_DVR_RealPlay_V40(lUserID, &struPlayInfo, NULL, NULL);
  if(checkResult(lRealPlayHandle<0,"start play error")){
    return HPR_ERROR;
  }
  int scRet;
  scRet = NET_DVR_SetRealDataCallBack(playHandle,fPlayDataCallBack<DWORD>,0);
  if(checkResult(scRet<0,"set play callback error")){
    return HPR_ERROR;
  }
  return 0;
}


int play(LPLoginInfo loginInfo, LPStream stream)
{
  lUserID = login(loginInfo);
  if(checkResult(lUserID<0, "device login error")){
    return HPR_ERROR;
  }
  int fChannelInfo = _readChannelConfig(lUserID, stream);
  if(fChannelInfo == HPR_ERROR){
    return fChannelInfo;
  }

  int ret = _getStream(lUserID, stream);
  if(ret != 0){
    cerr<<"Get stream error"<<endl;
    return HPR_ERROR;
  }
  while(true){
    sleep(10);
  }
  return HPR_OK;
}
